#include <atari.h>
#include <stdio.h>
#include <unistd.h>
#include "atari_multijoy.h"


/* FIXME: Will be part of atari.h's Page 0 labels */
#define SAVMSC (*(unsigned int*)0x58)

/* FIXME: Will be part of atari.h's Page 2 labels */
#define CRSINH (*(unsigned char*)0x2F0)
#define COLOR1 (*(unsigned char*)0x2C5)
#define COLOR2 (*(unsigned char*)0x2C6)
#define COLOR4 (*(unsigned char*)0x2C8)

/* FIXME: Will be part of atari.h */
unsigned char * PAGE6 = (char *) (0x600);


void dli_init(void) {
  unsigned char i;
  unsigned char * DLIST;
  unsigned char old_sdmctl;

  old_sdmctl = multijoy_dli_init_1();

  DLIST = (unsigned char *) SDLST;

  DLIST[3] = DL_DLI(DL_LMS(DL_GRAPHICS0));
  for (i = 6; i < 29; i++) {
    DLIST[i] = DL_DLI(DL_GRAPHICS0);
  }

  multijoy_dli_init_2(old_sdmctl);
}


int main(void) {
  unsigned char p, s, i, n, xx, yy, f, c;
  unsigned char life[4], x[4], y[4], b[16], bx[16], by[16], bs[16], old_fire_s[4], old_fire_s_ct[4];
  unsigned char * SC;

  multijoy_init(true);

  _graphics(0);
  CRSINH = 1;
  printf("%c", CH_CLR);
  COLOR1 = 15;
  COLOR2 = 0;

  dli_init();

  /* Obstacles: Middles of edges */
  SC = (char *) SAVMSC + 20;
  *SC = 128 + 32 + 26;

  SC = (char *) SAVMSC + 22 * 40 + 20;
  *SC = 128 + 32 + 26;

  SC = (char *) SAVMSC + 11 * 40;
  *SC = 128 + 32 + 26;

  SC = (char *) SAVMSC + 11 * 40 + 39;
  *SC = 128 + 32 + 26;

  /* Obstacles: Random in the middle */
  for (i = 0; i < 16; i++) {
    xx = (POKEY_READ.random & 0x1f) + 4;
    yy = (POKEY_READ.random % 23);
    SC = (char *) SAVMSC + yy * 40 + xx;
    *SC = 128 + 32 + 1 + (POKEY_READ.random % 10);

    xx = (POKEY_READ.random % 40);
    yy = (POKEY_READ.random & 0x0f) + 4;
    SC = (char *) SAVMSC + yy * 40 + xx;
    *SC = 128 + 32 + 1 + (POKEY_READ.random % 10);
  }

  /* Player starting positions: */
  for (p = 0; p < 4; p++) {
    life[p] = 8;
  }

  x[0] = 0;
  y[0] = 0;

  x[1] = 39;
  y[1] = 0;

  x[2] = 39;
  y[2] = 22;

  x[3] = 0;
  y[3] = 22;

  multijoy_wait();

  for (p = 0; p < 16; p++) {
    b[p] = false;
  }

  do {
    for (p = 0; p < 4; p++) {
      xx = x[p];
      yy = y[p];

      if (life[p]) {
        /* Alive player */

        /* Move player? */
        s = PAGE6[p << 1];
        if (s != 15) {
          SC = (char *) SAVMSC + yy * 40 + xx;
          *SC = 0;

          if (!(s & JOY_UP_MASK) && yy > 0) {
            yy--;
          } else if (!(s & JOY_DOWN_MASK) && yy < 22) {
            yy++;
          }
          if (!(s & JOY_LEFT_MASK) && xx > 0) {
            xx--;
          } else if (!(s & JOY_RIGHT_MASK) && xx < 39) {
            xx++;
          }

          SC = (char *) SAVMSC + yy * 40 + xx;
          if (*SC != 0) {
            xx = x[p];
            yy = y[p];
          }
        }

        SC = (char *) SAVMSC + yy * 40 + xx;
        *SC = p + 16;

        x[p] = xx;
        y[p] = yy;

        /* Fire? */
        s = PAGE6[(p << 1) + 1];
        if (s != 15 && s != old_fire_s[p]) {
          f = 17;
          for (i = 0; i < 4 && f == 17; i++) {
            n = (p << 2) + i;
            if (b[n] == false) {
              f = n;
            }
          }

          if (f != 17) {
            b[f] = true;
            bx[f] = xx;
            by[f] = yy;
            bs[f] = s;
          }
        }

        /* Keep track of previous firing direction, and allow for autofire */
        if (s == old_fire_s[p]) {
          old_fire_s_ct[p]++;
          if (old_fire_s_ct[p] == 8) {
            old_fire_s[p] = 15;
          }
        } else {
          old_fire_s[p] = s;
          old_fire_s_ct[p] = 0;
        }
      } else {
        /* Dead player */
        SC = (char *) SAVMSC + yy * 40 + xx;
        *SC = 0;
      }

      /* Show player's life meter */
      for (i = 0; i < 8; i++) {
        SC = (char *) (SAVMSC + (23 * 40) + (p * 10) + i + 1);
        if (i < life[p]) {
          *SC = p + 16 + 128;
        } else {
          *SC = p + 16;
        }
      }
    }

    /* Handle bullets */
    COLOR2 = 0;
    COLOR4 = 4;
    for (i = 0; i < 16; i++) {
      if (b[i]) {
        s = bs[i];

        xx = bx[i];
        yy = by[i];

        SC = (char *) SAVMSC + yy * 40 + xx;
        *SC = 0;

        if (!(s & JOY_UP_MASK)) {
          if (yy > 0) {
            yy--;
          } else {
            b[i] = false;
          }
        } else if (!(s & JOY_DOWN_MASK)) {
          if (yy < 22) {
            yy++;
          } else {
            b[i] = false;
          }
        }
        if (!(s & JOY_LEFT_MASK)) {
          if (xx > 0) {
            xx--;
          } else {
            b[i] = false;
          }
        } else if (!(s & JOY_RIGHT_MASK)) {
          if (xx < 39) {
            xx++;
          } else {
            b[i] = false;
          }
        }

        /* See if we hit an opponent */
        if (b[i]) {
          for (p = 0; p < 4; p++) {
            if (life[p] && x[p] == xx && y[p] == yy) {
              b[i] = false;
              COLOR4 = 15;
              life[p]--;
              if (life[p] == 0) {
                COLOR2 = 254;
              }
            }
          }
        }

        SC = (char *) SAVMSC + yy * 40 + xx;

        /* Check for obstacle */
        if (b[i]) {
          c = *SC;
          if (c >= 128) {
            c--;
            if (c == 128 + 32) {
              c = 0;
            }
            b[i] = false;

           *SC = c;
          }
        }

        /* If bullet still flying, draw it */
        if (b[i]) {
          *SC = 0x1A;

          bx[i] = xx;
          by[i] = yy;
        }
      }
    }
  } while (GTIA_READ.consol == 7);

  multijoy_init(false);

  return 0;
}
